#include "common.h"

#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <dirent.h>
#include <stddef.h>
#include <pthread.h>
#include <string.h>
#include <time.h>

#ifdef HAVE_LOCKS
#include <semaphore.h>
#endif

#include "ximc.h"
#include "util.h"
#include "platform.h"
#include "metadata.h"

#ifdef STRERROR_R_CHAR_P
#error GNU variant of strerror_r is not supported
#endif

#ifdef __APPLE__
/* We need IOKit */
#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/usb/IOUSBLib.h>
#endif


/*
 * Serial port support
 */

handle_t open_port (const char* name, unsigned int timeout)
{
	device_metadata_t dm;
	int fd;
	struct termios options;
	//printf("%s: entering name= %s\n", __func__,name);
	fd = open(name, O_RDWR | O_NOCTTY | O_NONBLOCK);
	//printf("%s: open port -> %d\n", __func__,fd);
	if (fd == -1)
	{
		log_system_error( L"unable to open port %s: ", name );
		printf("%s: unable to open port %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	/* Consult an advisory lock */

	if (flock( fd, LOCK_EX|LOCK_NB ) == -1 && errno == EWOULDBLOCK)
	{
		close( fd );
		log_error( L"unable to open locked port %s", name );
		printf("%s: unable to open locked port %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	if (flock( fd, LOCK_EX ) == -1)
	{
		log_system_error( L"can't lock file" );
		close( fd );
		log_error( L"unable to lock a port %s", name );
		printf("%s: unable to lock a port %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}


	/* Adjust settings */

	if (fcntl(fd, F_SETFL, 0) == -1)
	{
		close( fd );
		log_system_error( L"error setting port settings: " );
		printf("%s: error setting port settings: %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	if (tcgetattr(fd, &options) == -1)
	{
		close( fd );
		log_system_error( L"error getting port attrs: " );
		printf("%s: error getting port attrs: %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	if (cfsetispeed( &options, B115200 ) == -1 ||
		cfsetospeed( &options, B115200 ) == -1)
	{
		close( fd );
		log_system_error( L"error setting port speed: " );
		printf("%s: error setting port speed: %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	// set port flags
	options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	options.c_cflag |= (CLOCAL | CREAD);
 	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_cflag &= ~(PARENB | PARODD);
	options.c_cflag |= CSTOPB;
	options.c_cflag &= ~CRTSCTS;

	options.c_iflag &= ~(IXON | IXOFF | IXANY);
	options.c_iflag &= ~(INPCK | PARMRK | ISTRIP | IGNPAR);
	options.c_iflag &= ~(IGNBRK | BRKINT | INLCR | IGNCR | ICRNL | IMAXBEL);

	options.c_oflag &= ~OPOST;

	options.c_cc[VMIN] = 0;
	options.c_cc[VTIME] = timeout/100;

	if (tcsetattr( fd, TCSAFLUSH, &options ) == -1)
	{
		close( fd );
		log_system_error( L"error setting port attrs: " );
		printf("%s: error setting port attrs: %s, err=%s\n", __func__,name, strerror(errno));
		return handle_invalid;
	}

	tcflush( fd, TCIOFLUSH );

	/* save metadata */
	dm.timeout = timeout;
	dm.pipe = false;
	dm.device_mutex = NULL;
	set_metadata( (device_t)fd, dm );

	return fd;
}

int close_port (handle_t handle)
{
	if (close( handle ) == -1)
	{
		log_system_error( L"error closing port: " );
		return result_serial_error;
	}
	return result_serial_ok;
}

int command_port_flush (handle_t handle)
{
	if (tcflush( handle, TCIOFLUSH ))
	{
		log_system_error( L"command flush port failed, reason: " );
		return result_serial_error;
	}
	return result_serial_ok;
}

int command_port_send (handle_t handle, const byte* command, size_t command_len)
{
	ssize_t n;
	size_t k, amount;
	int err;

	for (k = 0; k < command_len; )
	{
		if (command_len < k)
			return result_serial_error;
		amount = command_len - k;
		#ifdef DEBUG_TRACE
		log_debug( L"writing %d ... ", (int)amount ); dump_bytes( command+k, amount );
		#endif
		n = write( handle, command+k, amount );
		err = errno;
		if (n < 0)
		{
			log_system_error( L"write to port failed, write %d bytes instead of %d bytes, reason: ", n, amount );
			if (err == ENXIO)
				return result_serial_nodevice;
			if (tcflush( handle, TCIOFLUSH ))
			{
				if (err == ENXIO || err == EIO)
					return result_serial_nodevice;
				log_system_error( L"flush port failed, p1, reason: " );
			}
			return result_serial_error;
		}
		if (n == 0 && k < command_len)
		{
			#ifdef DEBUG_TRACE
			log_debug( L"no more bytes (%d left)... ", (int)(command_len-k) );
			#endif
			if (tcflush( handle, TCIOFLUSH ))
			{
				if (err == ENXIO || err == EIO)
					return result_serial_nodevice;
				log_system_error( L"flush port failed, p2, reason: " );
			}
			return result_serial_timeout;
		}

		k += n;
	}

	return result_serial_ok;
}

int command_port_receive (handle_t handle, byte* response, size_t response_len)
{
	ssize_t n;
	size_t k, amount;
	int err;

	for (k = 0; k < response_len; )
	{
		amount = response_len - k;
		if (response_len < k)
			return result_serial_error;

		n = read( handle, response+k, amount );
		err = errno;
		#ifdef DEBUG_TRACE
		log_debug( L"reading %d/%d ... ", (int)n, (int)amount ); dump_bytes( response+k, ximc_max( 0, n ) );
		#endif
		if (n < 0)
		{
			log_system_error( L"read from port failed, read %d bytes instead of %d, reason: ", n, amount );
			if (err == ENXIO)
				return result_serial_nodevice;
			if (tcflush( handle, TCIOFLUSH ))
			{
				if (err == ENXIO || err == EIO)
					return result_serial_nodevice;
				log_system_error( L"flush port failed, p1, reason: " );
				return result_serial_error;
			}
			return result_serial_error;
		}
		if (n == 0 && k < response_len)
		{
			#ifdef DEBUG_TRACE
			log_debug( L"no more bytes (%d left)... ", (int)(response_len-k) );
			#endif
			if (tcflush( handle, TCIOFLUSH ))
			{
				if (err == ENXIO || err == EIO)
					return result_serial_nodevice;
				log_system_error( L"flush port failed, p2, reason: " );
				return result_serial_error;
			}
			return result_serial_timeout;
		}

		k += n;
	}

	return result_serial_ok;
}


/*
 * Threading support
 */

/* Internal carry struct for thread function */
typedef struct fork_join_carry_t
{
	fork_join_thread_function_t function;
	void* arg;
} fork_join_carry_t;

/* posix wrapper thread function */
void* check_thread_wrapper_posix (void *arg)
{
	fork_join_carry_t* carry = (fork_join_carry_t*)arg;
	carry->function( carry->arg );
	return NULL;
}

/* posix implementation of fork/join */
result_t fork_join (fork_join_thread_function_t function, int count, void* args, size_t arg_element_size)
{
	result_t result = result_ok;
	pthread_t* tids;
	pthread_attr_t thread_attr;
	int i;
	fork_join_carry_t* carry;

	pthread_attr_init( &thread_attr );
	pthread_attr_setdetachstate( &thread_attr, PTHREAD_CREATE_JOINABLE );

	tids = (pthread_t*)malloc( count*sizeof(pthread_t) );
	carry = (fork_join_carry_t*)malloc( count*sizeof(fork_join_carry_t) );

	/* launch and join */
	for (i = 0; i < count; ++i)
	{
		carry[i].function = function;
		carry[i].arg = (byte*)args + i*arg_element_size;
		if (pthread_create( &tids[i], &thread_attr, &check_thread_wrapper_posix, &carry[i] ))
		{
			result = result_error;
			log_system_error( L"Failed to create a pthread due to: " );
		}
	}
	for (i = 0; i < count; ++i)
	{
		if (pthread_join( tids[i], NULL ))
		{
			result = result_error;
			log_system_error( L"Failed to join a pthread due to: " );
		}
	}

	free( tids );
	free( carry );

	return result;
}

unsigned long long get_thread_id()
{
	return (unsigned long long)(uintptr_t)pthread_self();
}

/*
 * Device enumeration support
 */

// ftdi chip under generic unix has name /dev/ttyUSB*, under OS X /dev/tty.usbserial-*
// should we use generic com port name? - /dev/tty
int like_com_device_by_prefix (const char* prefix, const char* name)
{
	return strlen( name ) > strlen( prefix ) && !memcmp( prefix, name, strlen( prefix ) );
}

#ifdef __APPLE__
// mac os x
// Names are given by VCP driver by vendor ids
// xidc: Vendor ID 0x0403, /dev/tty.usbserial-* where /x+/ is sn
// xism: Vendor ID 0x1cbe, not listed, generic name /dev/tty.usbmodem*
// path must be slash-terminated
bool is_device_name_ok (char* directory, char* name, int flags)
{
	XIMC_UNUSED(directory);
	XIMC_UNUSED(flags);
	return
		!strcmp( directory, "/dev" ) && (
			like_com_device_by_prefix( "tty.usbserial-", name ) ||
			like_com_device_by_prefix( "tty.usbmodem", name )
		);
}
#else
// generic unix
bool is_device_name_ok (char* directory, char* name, int flags)
{
	return
		(!strcmp( directory, "/dev" ) && (flags & ENUMERATE_ALL_COM) && (
				like_com_device_by_prefix( "ttyUSB", name ) ||
				like_com_device_by_prefix( "ttyACM", name )))
		||
		(!strcmp( directory, "/dev/ximc" ) &&
		 		is_hex( name ));
}
#endif

#ifdef __APPLE__
/* directory must not end with slash */
result_t enumerate_iokit (enumerate_devices_directory_callback_t callback, void* arg, int flags)
{
	const long usbVendor = 0x1CBE;
	const long usbProduct = 0x0007;
	CFMutableDictionaryRef matchingDict;
	CFNumberRef numberRef;
	kern_return_t kr;
	io_iterator_t ioIterator;
	io_service_t usbDevice;
	CFStringRef refPath;
	char cpath[PATH_MAX];

	XIMC_UNUSED(flags);

	// IOUSBDevice and its subclasses
	matchingDict = IOServiceMatching(kIOUSBDeviceClassName);
	if (matchingDict == NULL)
		return result_error;

	// Create a CFNumber for the idVendor, idProduct and set the value in the dictionary
	numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbVendor);
	CFDictionarySetValue(matchingDict, CFSTR(kUSBVendorID), numberRef);
	CFRelease(numberRef);

	numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbProduct);
	CFDictionarySetValue(matchingDict, CFSTR(kUSBProductID), numberRef);
	CFRelease(numberRef);

	if ((kr = IOServiceGetMatchingServices(kIOMasterPortDefault, matchingDict, &ioIterator)) == KERN_SUCCESS)
	{
		while ((usbDevice = IOIteratorNext(ioIterator)) != 0)
		{
			if ((refPath = IORegistryEntrySearchCFProperty(usbDevice, kIOServicePlane, 
					CFSTR("IODialinDevice"), kCFAllocatorDefault, kIORegistryIterateRecursively)) != NULL)
			{
				CFStringGetCString(refPath, cpath, sizeof(cpath), kCFStringEncodingASCII);
				cpath[sizeof(cpath)-1] = 0;
				log_debug( L"Look to device %hs", cpath );
				/* device name is okay by default */
				callback( cpath, arg );
				CFRelease(refPath);
			}
			IOObjectRelease(usbDevice);
		}
	}
	return result_ok;
}

#endif

bool is_same_device (const char* name1, const char* name2)
{
	char realname1[PATH_MAX], realname2[PATH_MAX];
	if (!realpath( name1, realname1 ))
	{
		log_system_error( L"Cannot resolve real path of device %hs", name1 );
		return false;
	}
	if (!realpath( name2, realname2 ))
	{
		log_system_error( L"Cannot resolve real path of device %hs", name2 );
		return false;
	}
	return !strcmp( realname1, realname2 );
}

/* directory must not end with slash */
result_t enumerate_specific_directory (char* directory, enumerate_devices_directory_callback_t callback, void* arg, int flags)
{
	result_t result = result_ok;
	DIR* dev_dir;
	struct dirent* de;
	struct dirent* de_result;
	size_t de_size = 0;
	struct stat stat_buf;
	char full_path[PATH_MAX];

	/* do not fail if directory does not exist */
	if (stat( directory, &stat_buf ) == -1 && (errno == ENOENT || errno == ENOTDIR))
		return result_ok;

	if (!directory[0] || directory[strlen( directory )-1] == '/')
		return result_error;

	log_debug( L"Enumerating specific directory %hs", directory );

	/* Open a dir */
	dev_dir = opendir( directory );

	if (!dev_dir)
	{
		log_system_error( L"Can't open %hs dir due to: ", directory );
		return result_error;
	}

	de_size = offsetof(struct dirent, d_name) + pathconf(directory, _PC_NAME_MAX) + 1;
    de = (struct dirent*)malloc( de_size );

	for (;;)
	{
		if (readdir_r( dev_dir, de, &de_result) != 0)
		{
			log_system_error( L"Can't readdir %hs dir due to: ", directory );
			break;
		}

		/* No more entries */
		if (de_result == NULL)
			break;

		portable_snprintf( full_path, sizeof(full_path), "%s/%s", directory, de->d_name );
		full_path[sizeof(full_path)-1] = 0;

		/* Get statistic of file (resolved path for symlink) */
		if (stat( full_path, &stat_buf ) == -1)
		{
			if (errno == ENOENT || errno == ENOTDIR)
			{
				log_warning( L"File does not exist: %hs", full_path );
				continue;
			}
			else
			{
				log_system_error( L"Error accessing file: %hs", full_path );
				return result_error;
			}
		}
		/* Skip nondevice file */
		if (!S_ISCHR(stat_buf.st_mode))
			continue;

		log_debug( L"Look to device %hs", de->d_name );
		// check only name of the device
		if (is_device_name_ok( directory, de->d_name, flags ))
		{
			callback( full_path, arg );
		}
		else
			log_debug( L"Skip port %hs/%hs", directory, de->d_name );
	}

	free( de );
	closedir( dev_dir );

	return result;
}

result_t enumerate_devices_directory (enumerate_devices_directory_callback_t callback, void* arg, int flags)
{
	result_t result;

	#ifdef __APPLE__
	if (!(flags & ENUMERATE_ALL_COM))
	{
		/* use IOKit if only specific USB devices are needed */
		return enumerate_iokit( callback, arg, flags );
	}
	#endif

	/* enumerate /dev/ximc/ first */
	if ((result = enumerate_specific_directory( "/dev/ximc", callback, arg, flags )) != result_ok)
		return result;

 	/* enumerate all other devices in /dev/ because there are symlinks to them */
	if ((result = enumerate_specific_directory( "/dev", callback, arg, flags )) != result_ok)
		return result;

	return result_ok;
}

/*
 * Error handling
 */

unsigned int get_system_error_code ()
{
	return errno;
}

char* strerror_r_ensure (int code)
{
	size_t buflen = 256;
	char* buf = malloc( buflen );
	int ret;

	do
	{
		ret = strerror_r( code, buf, buflen );
		if (ret == ERANGE)
		{
			buflen = (size_t)(buflen * 1.4);
			free( buf );
			buf = malloc( buflen );
		}
	} while (!(ret == 0 || ret == EINVAL));

	return buf;
}

wchar_t* get_system_error_str (int code)
{
	char* str;
	wchar_t* result;
	str = strerror_r_ensure( code );
	if (!str)
		str = portable_strdup( "strerror_r failed, unknown error" );
	result = str_to_widestr( str );
	free( str );
	return result;
}

void free_system_error_str (wchar_t* str)
{
	if (str)
		free( str );
}

/*
 * Misc
 */

/* Dummy, windows-only */
int fix_usbser_sys(const char* device_name)
{
	XIMC_UNUSED(device_name);
	return 0;
}

void XIMC_API msec_sleep(unsigned int msec)
{
	// POSIX 1.b
	struct timespec ts;
	ts.tv_sec = (time_t)(msec / 1E3);
	ts.tv_nsec = (long)(msec*1E6 - ts.tv_sec*1E9);
	if (nanosleep( &ts, NULL ) != 0)
		log_system_error( L"nanosleep failed" );
}

void get_wallclock(time_t* sec, int* msec)
{
	struct timeval now;
	gettimeofday( &now, 0 );
	if (sec && msec)
	{
		*sec = now.tv_sec;
		*msec = now.tv_usec / 1000;
	}
}

/*
 * Lock support
 */

#ifdef HAVE_LOCKS

struct mutex_t
{
	sem_t* impl;
};

mutex_t* mutex_init(unsigned int nonce)
{
	char name[_POSIX_PATH_MAX];
	int counter = 0;
	mutex_t* mutex = malloc( sizeof(mutex_t) );
	if (!mutex)
	{
		log_system_error( L"can't create semaphore" );
		return NULL;
	}
	/* thanks APU for this idea */
	do
	{
  		portable_snprintf( name, sizeof(name), "/sem-ximc-%ld.%x.%d", (long)getpid(), nonce, counter++ );
		mutex->impl = sem_open( name, O_CREAT|O_EXCL, 0777, 1 );
	}
	while (mutex->impl == SEM_FAILED && errno == EEXIST);
	if (mutex->impl == SEM_FAILED)
	{
		free( mutex );
		log_system_error( L"can't create semaphore" );
		return NULL;
	}
	/* unlink early */
	if (sem_unlink( name ))
	{
		free( mutex );
		log_system_error( L"can't unlink semaphore" );
		return NULL;
	}
	log_debug( L"semaphore %p (%s) inited", mutex->impl, name );
	return mutex;
}

void mutex_close(mutex_t* mutex)
{
	if (mutex)
	{
 		/* already uninked
		 if (mutex->impl != SEM_FAILED)
		{
			if (sem_close( mutex->impl ) != 0)
				log_system_error( L"can't close semaphore due to " );
		}*/
		log_debug( L"semaphore %p closed", mutex->impl );
		free( mutex );
	}
}

void mutex_lock(mutex_t* mutex)
{
	if (!mutex || mutex->impl == SEM_FAILED)
	{
		log_error( L"no semaphore specified" );
		return;
	}
	if (sem_wait( mutex->impl ) == -1)
		log_system_error( L"can't wait on semaphore %p due to ", mutex->impl );
}

void mutex_unlock(mutex_t* mutex)
{
	if (!mutex || mutex->impl == SEM_FAILED)
	{
		log_error( L"no semaphore specified" );
		return;
	}
	if (sem_post( mutex->impl ) == -1)
		log_system_error( L"can't post on semaphore %p due to ", mutex->impl );
}
#endif

// vim: syntax=c tabstop=4 shiftwidth=4
