#include "common.h"

#include "ximc.h"
#include "util.h"
#include "platform.h"
#include "protosup.h"
#include "loader.h"
#include "ximc-gen.h"

#if !defined(LIBXIMC_EXPORTS)
#error LIBXIMC_EXPORTS must not be defined when building a library
#endif

#if defined(__cplusplus)
extern "C" {
#endif


/*
 -------------------------------------------
   BEGIN OF GENERATED function definitions
 -------------------------------------------
*/
result_t XIMC_API set_feedback_settings (device_t id, const feedback_settings_t* feedback_settings)
{
	result_t result;
	byte command[18], *p  = command;

	lock( id );

	push_str( &p, "sfbs" );
	push_uint16( &p, feedback_settings->IPS );
	push_uint8( &p, feedback_settings->FeedbackType );
	push_uint8( &p, feedback_settings->FeedbackFlags );
	push_uint16( &p, feedback_settings->HallSPR );
	push_int8( &p, feedback_settings->HallShift );
	push_garbage( &p, 5 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_feedback_settings (device_t id, feedback_settings_t* feedback_settings)
{
	result_t result;
	byte response[18], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gfbs", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	feedback_settings->IPS = pop_uint16( &p );
	feedback_settings->FeedbackType = pop_uint8( &p );
	feedback_settings->FeedbackFlags = pop_uint8( &p );
	feedback_settings->HallSPR = pop_uint16( &p );
	feedback_settings->HallShift = pop_int8( &p );
	pop_garbage( &p, 5 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_home_settings (device_t id, const home_settings_t* home_settings)
{
	result_t result;
	byte command[33], *p  = command;

	lock( id );

	push_str( &p, "shom" );
	push_uint32( &p, home_settings->FastHome );
	push_uint8( &p, home_settings->uFastHome );
	push_uint32( &p, home_settings->SlowHome );
	push_uint8( &p, home_settings->uSlowHome );
	push_int32( &p, home_settings->HomeDelta );
	push_int16( &p, home_settings->uHomeDelta );
	push_uint16( &p, home_settings->HomeFlags );
	push_garbage( &p, 9 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_home_settings_calb (device_t id, const home_settings_calb_t* home_settings_calb, const calibration_t* calibration)
{
	home_settings_t inner;
	XI_calibrate_to_normal(home_settings_calb->FastHome, inner.FastHome, inner.uFastHome, calibration);
	XI_calibrate_to_normal(home_settings_calb->SlowHome, inner.SlowHome, inner.uSlowHome, calibration);
	XI_calibrate_to_normal(home_settings_calb->HomeDelta, inner.HomeDelta, inner.uHomeDelta, calibration);
	inner.HomeFlags = home_settings_calb->HomeFlags;

	return set_home_settings(id, &inner);
}

result_t XIMC_API get_home_settings (device_t id, home_settings_t* home_settings)
{
	result_t result;
	byte response[33], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "ghom", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	home_settings->FastHome = pop_uint32( &p );
	home_settings->uFastHome = pop_uint8( &p );
	home_settings->SlowHome = pop_uint32( &p );
	home_settings->uSlowHome = pop_uint8( &p );
	home_settings->HomeDelta = pop_int32( &p );
	home_settings->uHomeDelta = pop_int16( &p );
	home_settings->HomeFlags = pop_uint16( &p );
	pop_garbage( &p, 9 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_home_settings_calb (device_t id, home_settings_calb_t* home_settings_calb, const calibration_t* calibration)
{
	result_t result;
	home_settings_t inner;

	if ((result = get_home_settings(id, &inner)) != result_ok)
		return result;

	XI_normal_to_calibrate(home_settings_calb->FastHome, inner.FastHome, inner.uFastHome, calibration);
	XI_normal_to_calibrate(home_settings_calb->SlowHome, inner.SlowHome, inner.uSlowHome, calibration);
	XI_normal_to_calibrate(home_settings_calb->HomeDelta, inner.HomeDelta, inner.uHomeDelta, calibration);
	home_settings_calb->HomeFlags = inner.HomeFlags;

	return result;
}

result_t XIMC_API set_move_settings (device_t id, const move_settings_t* move_settings)
{
	result_t result;
	byte command[30], *p  = command;

	lock( id );

	push_str( &p, "smov" );
	push_uint32( &p, move_settings->Speed );
	push_uint8( &p, move_settings->uSpeed );
	push_uint16( &p, move_settings->Accel );
	push_uint16( &p, move_settings->Decel );
	push_uint32( &p, move_settings->AntiplaySpeed );
	push_uint8( &p, move_settings->uAntiplaySpeed );
	push_garbage( &p, 10 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_move_settings_calb (device_t id, const move_settings_calb_t* move_settings_calb, const calibration_t* calibration)
{
	move_settings_t inner;
	XI_calibrate_to_normal(move_settings_calb->Speed, inner.Speed, inner.uSpeed, calibration);
	XI_calibrate_to_normal_short(move_settings_calb->Accel, inner.Accel, calibration);
	XI_calibrate_to_normal_short(move_settings_calb->Decel, inner.Decel, calibration);
	XI_calibrate_to_normal(move_settings_calb->AntiplaySpeed, inner.AntiplaySpeed, inner.uAntiplaySpeed, calibration);

	return set_move_settings(id, &inner);
}

result_t XIMC_API get_move_settings (device_t id, move_settings_t* move_settings)
{
	result_t result;
	byte response[30], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gmov", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	move_settings->Speed = pop_uint32( &p );
	move_settings->uSpeed = pop_uint8( &p );
	move_settings->Accel = pop_uint16( &p );
	move_settings->Decel = pop_uint16( &p );
	move_settings->AntiplaySpeed = pop_uint32( &p );
	move_settings->uAntiplaySpeed = pop_uint8( &p );
	pop_garbage( &p, 10 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_move_settings_calb (device_t id, move_settings_calb_t* move_settings_calb, const calibration_t* calibration)
{
	result_t result;
	move_settings_t inner;

	if ((result = get_move_settings(id, &inner)) != result_ok)
		return result;

	XI_normal_to_calibrate(move_settings_calb->Speed, inner.Speed, inner.uSpeed, calibration);
	XI_normal_to_calibrate_short(move_settings_calb->Accel, inner.Accel, calibration);
	XI_normal_to_calibrate_short(move_settings_calb->Decel, inner.Decel, calibration);
	XI_normal_to_calibrate(move_settings_calb->AntiplaySpeed, inner.AntiplaySpeed, inner.uAntiplaySpeed, calibration);

	return result;
}

result_t XIMC_API set_engine_settings (device_t id, const engine_settings_t* engine_settings)
{
	result_t result;
	byte command[34], *p  = command;

	lock( id );

	push_str( &p, "seng" );
	push_uint16( &p, engine_settings->NomVoltage );
	push_uint16( &p, engine_settings->NomCurrent );
	push_uint32( &p, engine_settings->NomSpeed );
	push_uint8( &p, engine_settings->uNomSpeed );
	push_uint16( &p, engine_settings->EngineFlags );
	push_int16( &p, engine_settings->Antiplay );
	push_uint8( &p, engine_settings->MicrostepMode );
	push_uint16( &p, engine_settings->StepsPerRev );
	push_garbage( &p, 12 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_engine_settings_calb (device_t id, const engine_settings_calb_t* engine_settings_calb, const calibration_t* calibration)
{
	engine_settings_t inner;
	inner.NomVoltage = engine_settings_calb->NomVoltage;
	inner.NomCurrent = engine_settings_calb->NomCurrent;
	XI_calibrate_to_normal(engine_settings_calb->NomSpeed, inner.NomSpeed, inner.uNomSpeed, calibration);
	inner.EngineFlags = engine_settings_calb->EngineFlags;
	XI_calibrate_to_normal_short(engine_settings_calb->Antiplay, inner.Antiplay, calibration);
	inner.MicrostepMode = engine_settings_calb->MicrostepMode;
	inner.StepsPerRev = engine_settings_calb->StepsPerRev;

	return set_engine_settings(id, &inner);
}

result_t XIMC_API get_engine_settings (device_t id, engine_settings_t* engine_settings)
{
	result_t result;
	byte response[34], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "geng", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	engine_settings->NomVoltage = pop_uint16( &p );
	engine_settings->NomCurrent = pop_uint16( &p );
	engine_settings->NomSpeed = pop_uint32( &p );
	engine_settings->uNomSpeed = pop_uint8( &p );
	engine_settings->EngineFlags = pop_uint16( &p );
	engine_settings->Antiplay = pop_int16( &p );
	engine_settings->MicrostepMode = pop_uint8( &p );
	engine_settings->StepsPerRev = pop_uint16( &p );
	pop_garbage( &p, 12 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_engine_settings_calb (device_t id, engine_settings_calb_t* engine_settings_calb, const calibration_t* calibration)
{
	result_t result;
	engine_settings_t inner;

	if ((result = get_engine_settings(id, &inner)) != result_ok)
		return result;

	engine_settings_calb->NomVoltage = inner.NomVoltage;
	engine_settings_calb->NomCurrent = inner.NomCurrent;
	XI_normal_to_calibrate(engine_settings_calb->NomSpeed, inner.NomSpeed, inner.uNomSpeed, calibration);
	engine_settings_calb->EngineFlags = inner.EngineFlags;
	XI_normal_to_calibrate_short(engine_settings_calb->Antiplay, inner.Antiplay, calibration);
	engine_settings_calb->MicrostepMode = inner.MicrostepMode;
	engine_settings_calb->StepsPerRev = inner.StepsPerRev;

	return result;
}

result_t XIMC_API set_entype_settings (device_t id, const entype_settings_t* entype_settings)
{
	result_t result;
	byte command[14], *p  = command;

	lock( id );

	push_str( &p, "sent" );
	push_uint8( &p, entype_settings->EngineType );
	push_uint8( &p, entype_settings->DriverType );
	push_garbage( &p, 6 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_entype_settings (device_t id, entype_settings_t* entype_settings)
{
	result_t result;
	byte response[14], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gent", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	entype_settings->EngineType = pop_uint8( &p );
	entype_settings->DriverType = pop_uint8( &p );
	pop_garbage( &p, 6 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_power_settings (device_t id, const power_settings_t* power_settings)
{
	result_t result;
	byte command[20], *p  = command;

	lock( id );

	push_str( &p, "spwr" );
	push_uint8( &p, power_settings->HoldCurrent );
	push_uint16( &p, power_settings->CurrReductDelay );
	push_uint16( &p, power_settings->PowerOffDelay );
	push_uint16( &p, power_settings->CurrentSetTime );
	push_uint8( &p, power_settings->PowerFlags );
	push_garbage( &p, 6 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_power_settings (device_t id, power_settings_t* power_settings)
{
	result_t result;
	byte response[20], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gpwr", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	power_settings->HoldCurrent = pop_uint8( &p );
	power_settings->CurrReductDelay = pop_uint16( &p );
	power_settings->PowerOffDelay = pop_uint16( &p );
	power_settings->CurrentSetTime = pop_uint16( &p );
	power_settings->PowerFlags = pop_uint8( &p );
	pop_garbage( &p, 6 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_secure_settings (device_t id, const secure_settings_t* secure_settings)
{
	result_t result;
	byte command[28], *p  = command;

	lock( id );

	push_str( &p, "ssec" );
	push_uint16( &p, secure_settings->LowUpwrOff );
	push_uint16( &p, secure_settings->CriticalIpwr );
	push_uint16( &p, secure_settings->CriticalUpwr );
	push_uint16( &p, secure_settings->CriticalT );
	push_uint16( &p, secure_settings->CriticalIusb );
	push_uint16( &p, secure_settings->CriticalUusb );
	push_uint16( &p, secure_settings->MinimumUusb );
	push_uint8( &p, secure_settings->Flags );
	push_garbage( &p, 7 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_secure_settings (device_t id, secure_settings_t* secure_settings)
{
	result_t result;
	byte response[28], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gsec", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	secure_settings->LowUpwrOff = pop_uint16( &p );
	secure_settings->CriticalIpwr = pop_uint16( &p );
	secure_settings->CriticalUpwr = pop_uint16( &p );
	secure_settings->CriticalT = pop_uint16( &p );
	secure_settings->CriticalIusb = pop_uint16( &p );
	secure_settings->CriticalUusb = pop_uint16( &p );
	secure_settings->MinimumUusb = pop_uint16( &p );
	secure_settings->Flags = pop_uint8( &p );
	pop_garbage( &p, 7 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_edges_settings (device_t id, const edges_settings_t* edges_settings)
{
	result_t result;
	byte command[26], *p  = command;

	lock( id );

	push_str( &p, "seds" );
	push_uint8( &p, edges_settings->BorderFlags );
	push_uint8( &p, edges_settings->EnderFlags );
	push_int32( &p, edges_settings->LeftBorder );
	push_int16( &p, edges_settings->uLeftBorder );
	push_int32( &p, edges_settings->RightBorder );
	push_int16( &p, edges_settings->uRightBorder );
	push_garbage( &p, 6 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_edges_settings_calb (device_t id, const edges_settings_calb_t* edges_settings_calb, const calibration_t* calibration)
{
	edges_settings_t inner;
	inner.BorderFlags = edges_settings_calb->BorderFlags;
	inner.EnderFlags = edges_settings_calb->EnderFlags;
	XI_calibrate_to_normal(edges_settings_calb->LeftBorder, inner.LeftBorder, inner.uLeftBorder, calibration);
	XI_calibrate_to_normal(edges_settings_calb->RightBorder, inner.RightBorder, inner.uRightBorder, calibration);

	return set_edges_settings(id, &inner);
}

result_t XIMC_API get_edges_settings (device_t id, edges_settings_t* edges_settings)
{
	result_t result;
	byte response[26], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "geds", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	edges_settings->BorderFlags = pop_uint8( &p );
	edges_settings->EnderFlags = pop_uint8( &p );
	edges_settings->LeftBorder = pop_int32( &p );
	edges_settings->uLeftBorder = pop_int16( &p );
	edges_settings->RightBorder = pop_int32( &p );
	edges_settings->uRightBorder = pop_int16( &p );
	pop_garbage( &p, 6 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_edges_settings_calb (device_t id, edges_settings_calb_t* edges_settings_calb, const calibration_t* calibration)
{
	result_t result;
	edges_settings_t inner;

	if ((result = get_edges_settings(id, &inner)) != result_ok)
		return result;

	edges_settings_calb->BorderFlags = inner.BorderFlags;
	edges_settings_calb->EnderFlags = inner.EnderFlags;
	XI_normal_to_calibrate(edges_settings_calb->LeftBorder, inner.LeftBorder, inner.uLeftBorder, calibration);
	XI_normal_to_calibrate(edges_settings_calb->RightBorder, inner.RightBorder, inner.uRightBorder, calibration);

	return result;
}

result_t XIMC_API set_pid_settings (device_t id, const pid_settings_t* pid_settings)
{
	result_t result;
	byte command[48], *p  = command;

	lock( id );

	push_str( &p, "spid" );
	push_uint16( &p, pid_settings->KpU );
	push_uint16( &p, pid_settings->KiU );
	push_uint16( &p, pid_settings->KdU );
	push_garbage( &p, 36 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_pid_settings (device_t id, pid_settings_t* pid_settings)
{
	result_t result;
	byte response[48], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gpid", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pid_settings->KpU = pop_uint16( &p );
	pid_settings->KiU = pop_uint16( &p );
	pid_settings->KdU = pop_uint16( &p );
	pop_garbage( &p, 36 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_sync_in_settings (device_t id, const sync_in_settings_t* sync_in_settings)
{
	result_t result;
	byte command[28], *p  = command;

	lock( id );

	push_str( &p, "ssni" );
	push_uint8( &p, sync_in_settings->SyncInFlags );
	push_uint16( &p, sync_in_settings->ClutterTime );
	push_int32( &p, sync_in_settings->Position );
	push_int16( &p, sync_in_settings->uPosition );
	push_uint32( &p, sync_in_settings->Speed );
	push_uint8( &p, sync_in_settings->uSpeed );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_sync_in_settings_calb (device_t id, const sync_in_settings_calb_t* sync_in_settings_calb, const calibration_t* calibration)
{
	sync_in_settings_t inner;
	inner.SyncInFlags = sync_in_settings_calb->SyncInFlags;
	inner.ClutterTime = sync_in_settings_calb->ClutterTime;
	XI_calibrate_to_normal(sync_in_settings_calb->Position, inner.Position, inner.uPosition, calibration);
	XI_calibrate_to_normal(sync_in_settings_calb->Speed, inner.Speed, inner.uSpeed, calibration);

	return set_sync_in_settings(id, &inner);
}

result_t XIMC_API get_sync_in_settings (device_t id, sync_in_settings_t* sync_in_settings)
{
	result_t result;
	byte response[28], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gsni", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	sync_in_settings->SyncInFlags = pop_uint8( &p );
	sync_in_settings->ClutterTime = pop_uint16( &p );
	sync_in_settings->Position = pop_int32( &p );
	sync_in_settings->uPosition = pop_int16( &p );
	sync_in_settings->Speed = pop_uint32( &p );
	sync_in_settings->uSpeed = pop_uint8( &p );
	pop_garbage( &p, 8 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_sync_in_settings_calb (device_t id, sync_in_settings_calb_t* sync_in_settings_calb, const calibration_t* calibration)
{
	result_t result;
	sync_in_settings_t inner;

	if ((result = get_sync_in_settings(id, &inner)) != result_ok)
		return result;

	sync_in_settings_calb->SyncInFlags = inner.SyncInFlags;
	sync_in_settings_calb->ClutterTime = inner.ClutterTime;
	XI_normal_to_calibrate(sync_in_settings_calb->Position, inner.Position, inner.uPosition, calibration);
	XI_normal_to_calibrate(sync_in_settings_calb->Speed, inner.Speed, inner.uSpeed, calibration);

	return result;
}

result_t XIMC_API set_sync_out_settings (device_t id, const sync_out_settings_t* sync_out_settings)
{
	result_t result;
	byte command[16], *p  = command;

	lock( id );

	push_str( &p, "ssno" );
	push_uint8( &p, sync_out_settings->SyncOutFlags );
	push_uint16( &p, sync_out_settings->SyncOutPulseSteps );
	push_uint16( &p, sync_out_settings->SyncOutPeriod );
	push_uint32( &p, sync_out_settings->Accuracy );
	push_uint8( &p, sync_out_settings->uAccuracy );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_sync_out_settings_calb (device_t id, const sync_out_settings_calb_t* sync_out_settings_calb, const calibration_t* calibration)
{
	sync_out_settings_t inner;
	inner.SyncOutFlags = sync_out_settings_calb->SyncOutFlags;
	inner.SyncOutPulseSteps = sync_out_settings_calb->SyncOutPulseSteps;
	inner.SyncOutPeriod = sync_out_settings_calb->SyncOutPeriod;
	XI_calibrate_to_normal(sync_out_settings_calb->Accuracy, inner.Accuracy, inner.uAccuracy, calibration);

	return set_sync_out_settings(id, &inner);
}

result_t XIMC_API get_sync_out_settings (device_t id, sync_out_settings_t* sync_out_settings)
{
	result_t result;
	byte response[16], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gsno", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	sync_out_settings->SyncOutFlags = pop_uint8( &p );
	sync_out_settings->SyncOutPulseSteps = pop_uint16( &p );
	sync_out_settings->SyncOutPeriod = pop_uint16( &p );
	sync_out_settings->Accuracy = pop_uint32( &p );
	sync_out_settings->uAccuracy = pop_uint8( &p );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_sync_out_settings_calb (device_t id, sync_out_settings_calb_t* sync_out_settings_calb, const calibration_t* calibration)
{
	result_t result;
	sync_out_settings_t inner;

	if ((result = get_sync_out_settings(id, &inner)) != result_ok)
		return result;

	sync_out_settings_calb->SyncOutFlags = inner.SyncOutFlags;
	sync_out_settings_calb->SyncOutPulseSteps = inner.SyncOutPulseSteps;
	sync_out_settings_calb->SyncOutPeriod = inner.SyncOutPeriod;
	XI_normal_to_calibrate(sync_out_settings_calb->Accuracy, inner.Accuracy, inner.uAccuracy, calibration);

	return result;
}

result_t XIMC_API set_extio_settings (device_t id, const extio_settings_t* extio_settings)
{
	result_t result;
	byte command[18], *p  = command;

	lock( id );

	push_str( &p, "seio" );
	push_uint8( &p, extio_settings->EXTIOSetupFlags );
	push_uint8( &p, extio_settings->EXTIOModeFlags );
	push_garbage( &p, 10 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_extio_settings (device_t id, extio_settings_t* extio_settings)
{
	result_t result;
	byte response[18], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "geio", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	extio_settings->EXTIOSetupFlags = pop_uint8( &p );
	extio_settings->EXTIOModeFlags = pop_uint8( &p );
	pop_garbage( &p, 10 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_brake_settings (device_t id, const brake_settings_t* brake_settings)
{
	result_t result;
	byte command[25], *p  = command;

	lock( id );

	push_str( &p, "sbrk" );
	push_uint16( &p, brake_settings->t1 );
	push_uint16( &p, brake_settings->t2 );
	push_uint16( &p, brake_settings->t3 );
	push_uint16( &p, brake_settings->t4 );
	push_uint8( &p, brake_settings->BrakeFlags );
	push_garbage( &p, 10 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_brake_settings (device_t id, brake_settings_t* brake_settings)
{
	result_t result;
	byte response[25], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gbrk", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	brake_settings->t1 = pop_uint16( &p );
	brake_settings->t2 = pop_uint16( &p );
	brake_settings->t3 = pop_uint16( &p );
	brake_settings->t4 = pop_uint16( &p );
	brake_settings->BrakeFlags = pop_uint8( &p );
	pop_garbage( &p, 10 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_control_settings (device_t id, const control_settings_t* control_settings)
{
	result_t result;
	byte command[93], *p  = command;
	unsigned int i;

	lock( id );

	push_str( &p, "sctl" );
	for (i = 0; i < 10; ++i)
		push_uint32( &p, control_settings->MaxSpeed[i] );
	for (i = 0; i < 10; ++i)
		push_uint8( &p, control_settings->uMaxSpeed[i] );
	for (i = 0; i < 9; ++i)
		push_uint16( &p, control_settings->Timeout[i] );
	push_uint16( &p, control_settings->MaxClickTime );
	push_uint16( &p, control_settings->Flags );
	push_int32( &p, control_settings->DeltaPosition );
	push_int16( &p, control_settings->uDeltaPosition );
	push_garbage( &p, 9 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_control_settings_calb (device_t id, const control_settings_calb_t* control_settings_calb, const calibration_t* calibration)
{
	control_settings_t inner;
	unsigned int i;
	for (i = 0; i < 10; ++i)
	{
		XI_calibrate_to_normal(control_settings_calb->MaxSpeed[i], inner.MaxSpeed[i], inner.uMaxSpeed[i], calibration);
	};
	for (i = 0; i < 9; ++i)
		inner.Timeout[i] = control_settings_calb->Timeout[i];
	inner.MaxClickTime = control_settings_calb->MaxClickTime;
	inner.Flags = control_settings_calb->Flags;
	XI_calibrate_to_normal(control_settings_calb->DeltaPosition, inner.DeltaPosition, inner.uDeltaPosition, calibration);

	return set_control_settings(id, &inner);
}

result_t XIMC_API get_control_settings (device_t id, control_settings_t* control_settings)
{
	result_t result;
	byte response[93], *p  = response;
	unsigned int i;

	lock( id );

	if ((result = command_checked_str( id, "gctl", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	for (i = 0; i < 10; ++i)
		control_settings->MaxSpeed[i] = pop_uint32( &p );
	for (i = 0; i < 10; ++i)
		control_settings->uMaxSpeed[i] = pop_uint8( &p );
	for (i = 0; i < 9; ++i)
		control_settings->Timeout[i] = pop_uint16( &p );
	control_settings->MaxClickTime = pop_uint16( &p );
	control_settings->Flags = pop_uint16( &p );
	control_settings->DeltaPosition = pop_int32( &p );
	control_settings->uDeltaPosition = pop_int16( &p );
	pop_garbage( &p, 9 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_control_settings_calb (device_t id, control_settings_calb_t* control_settings_calb, const calibration_t* calibration)
{
	result_t result;
	control_settings_t inner;
	unsigned int i;

	if ((result = get_control_settings(id, &inner)) != result_ok)
		return result;

	for (i = 0; i < 10; ++i)
	{
		XI_normal_to_calibrate(control_settings_calb->MaxSpeed[i], inner.MaxSpeed[i], inner.uMaxSpeed[i], calibration);
	};
	for (i = 0; i < 9; ++i)
		control_settings_calb->Timeout[i] = inner.Timeout[i];
	control_settings_calb->MaxClickTime = inner.MaxClickTime;
	control_settings_calb->Flags = inner.Flags;
	XI_normal_to_calibrate(control_settings_calb->DeltaPosition, inner.DeltaPosition, inner.uDeltaPosition, calibration);

	return result;
}

result_t XIMC_API set_joystick_settings (device_t id, const joystick_settings_t* joystick_settings)
{
	result_t result;
	byte command[22], *p  = command;

	lock( id );

	push_str( &p, "sjoy" );
	push_uint16( &p, joystick_settings->JoyLowEnd );
	push_uint16( &p, joystick_settings->JoyCenter );
	push_uint16( &p, joystick_settings->JoyHighEnd );
	push_uint8( &p, joystick_settings->ExpFactor );
	push_uint8( &p, joystick_settings->DeadZone );
	push_uint8( &p, joystick_settings->JoyFlags );
	push_garbage( &p, 7 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_joystick_settings (device_t id, joystick_settings_t* joystick_settings)
{
	result_t result;
	byte response[22], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gjoy", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	joystick_settings->JoyLowEnd = pop_uint16( &p );
	joystick_settings->JoyCenter = pop_uint16( &p );
	joystick_settings->JoyHighEnd = pop_uint16( &p );
	joystick_settings->ExpFactor = pop_uint8( &p );
	joystick_settings->DeadZone = pop_uint8( &p );
	joystick_settings->JoyFlags = pop_uint8( &p );
	pop_garbage( &p, 7 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_ctp_settings (device_t id, const ctp_settings_t* ctp_settings)
{
	result_t result;
	byte command[18], *p  = command;

	lock( id );

	push_str( &p, "sctp" );
	push_uint8( &p, ctp_settings->CTPMinError );
	push_uint8( &p, ctp_settings->CTPFlags );
	push_garbage( &p, 10 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_ctp_settings (device_t id, ctp_settings_t* ctp_settings)
{
	result_t result;
	byte response[18], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gctp", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	ctp_settings->CTPMinError = pop_uint8( &p );
	ctp_settings->CTPFlags = pop_uint8( &p );
	pop_garbage( &p, 10 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_uart_settings (device_t id, const uart_settings_t* uart_settings)
{
	result_t result;
	byte command[16], *p  = command;

	lock( id );

	push_str( &p, "surt" );
	push_uint32( &p, uart_settings->Speed );
	push_uint16( &p, uart_settings->UARTSetupFlags );
	push_garbage( &p, 4 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_uart_settings (device_t id, uart_settings_t* uart_settings)
{
	result_t result;
	byte response[16], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gurt", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	uart_settings->Speed = pop_uint32( &p );
	uart_settings->UARTSetupFlags = pop_uint16( &p );
	pop_garbage( &p, 4 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_controller_name (device_t id, const controller_name_t* controller_name)
{
	result_t result;
	byte command[30], *p  = command;

	lock( id );

	push_str( &p, "snmf" );
	push_data( &p, controller_name->ControllerName, 16 );
	push_uint8( &p, controller_name->CtrlFlags );
	push_garbage( &p, 7 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_controller_name (device_t id, controller_name_t* controller_name)
{
	result_t result;
	byte response[30], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gnmf", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, controller_name->ControllerName, 16 );
	controller_name->CtrlFlags = pop_uint8( &p );
	pop_garbage( &p, 7 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API command_stop (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "stop" ) );
}


result_t XIMC_API set_add_sync_in_action (device_t id, const add_sync_in_action_t* add_sync_in_action)
{
	result_t result;
	byte command[22], *p  = command;

	lock( id );

	push_str( &p, "asia" );
	push_int32( &p, add_sync_in_action->Position );
	push_int16( &p, add_sync_in_action->uPosition );
	push_uint32( &p, add_sync_in_action->Speed );
	push_uint8( &p, add_sync_in_action->uSpeed );
	push_garbage( &p, 5 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_add_sync_in_action_calb (device_t id, const add_sync_in_action_calb_t* add_sync_in_action_calb, const calibration_t* calibration)
{
	add_sync_in_action_t inner;
	XI_calibrate_to_normal(add_sync_in_action_calb->Position, inner.Position, inner.uPosition, calibration);
	XI_calibrate_to_normal(add_sync_in_action_calb->Speed, inner.Speed, inner.uSpeed, calibration);

	return set_add_sync_in_action(id, &inner);
}

result_t XIMC_API command_power_off (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "pwof" ) );
}


result_t XIMC_API command_move (device_t id, int Position, int uPosition)
{
	result_t result;
	byte command[18], *p  = command;

	lock( id );

	push_str( &p, "move" );
	push_int32( &p, Position );
	push_int16( &p, uPosition );
	push_garbage( &p, 6 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API command_move_calb (device_t id, float cPosition, const calibration_t* calibration)
{
	int Position;
	int uPosition;
	XI_calibrate_to_normal(cPosition, Position, uPosition, calibration);

	return command_move(id, Position, uPosition);
}

result_t XIMC_API command_movr (device_t id, int DeltaPosition, int uDeltaPosition)
{
	result_t result;
	byte command[18], *p  = command;

	lock( id );

	push_str( &p, "movr" );
	push_int32( &p, DeltaPosition );
	push_int16( &p, uDeltaPosition );
	push_garbage( &p, 6 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API command_movr_calb (device_t id, float cDeltaPosition, const calibration_t* calibration)
{
	int DeltaPosition;
	int uDeltaPosition;
	XI_calibrate_to_normal(cDeltaPosition, DeltaPosition, uDeltaPosition, calibration);

	return command_movr(id, DeltaPosition, uDeltaPosition);
}

result_t XIMC_API command_home (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "home" ) );
}


result_t XIMC_API command_left (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "left" ) );
}


result_t XIMC_API command_right (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "rigt" ) );
}


result_t XIMC_API command_loft (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "loft" ) );
}


result_t XIMC_API command_sstp (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "sstp" ) );
}


result_t XIMC_API get_position (device_t id, get_position_t* the_get_position)
{
	result_t result;
	byte response[26], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gpos", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	the_get_position->Position = pop_int32( &p );
	the_get_position->uPosition = pop_int16( &p );
	the_get_position->EncPosition = pop_int64( &p );
	pop_garbage( &p, 6 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_position_calb (device_t id, get_position_calb_t* the_get_position_calb, const calibration_t* calibration)
{
	result_t result;
	get_position_t inner;

	if ((result = get_position(id, &inner)) != result_ok)
		return result;

	XI_normal_to_calibrate(the_get_position_calb->Position, inner.Position, inner.uPosition, calibration);
	the_get_position_calb->EncPosition = inner.EncPosition;

	return result;
}

result_t XIMC_API set_position (device_t id, const set_position_t* the_set_position)
{
	result_t result;
	byte command[26], *p  = command;

	lock( id );

	push_str( &p, "spos" );
	push_int32( &p, the_set_position->Position );
	push_int16( &p, the_set_position->uPosition );
	push_int64( &p, the_set_position->EncPosition );
	push_uint8( &p, the_set_position->PosFlags );
	push_garbage( &p, 5 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API set_position_calb (device_t id, const set_position_calb_t* the_set_position_calb, const calibration_t* calibration)
{
	set_position_t inner;
	XI_calibrate_to_normal(the_set_position_calb->Position, inner.Position, inner.uPosition, calibration);
	inner.EncPosition = the_set_position_calb->EncPosition;
	inner.PosFlags = the_set_position_calb->PosFlags;

	return set_position(id, &inner);
}

result_t XIMC_API command_zero (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "zero" ) );
}


result_t XIMC_API command_save_settings (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "save" ) );
}


result_t XIMC_API command_read_settings (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "read" ) );
}


result_t XIMC_API command_eesave_settings (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "eesv" ) );
}


result_t XIMC_API command_eeread_settings (device_t id)
{
	lock( id );
	return unlocker( id, command_checked_echo_str( id, "eerd" ) );
}


result_t get_status_impl (device_t id, status_t* status)
{
	result_t result;
	byte response[54], *p  = response;

	if ((result = command_checked_str( id, "gets", response, sizeof(response) )) != result_ok)
		return result;
	p += 4;

	status->MoveSts = pop_uint8( &p );
	status->MvCmdSts = pop_uint8( &p );
	status->PWRSts = pop_uint8( &p );
	status->EncSts = pop_uint8( &p );
	status->WindSts = pop_uint8( &p );
	status->CurPosition = pop_int32( &p );
	status->uCurPosition = pop_int16( &p );
	status->EncPosition = pop_int64( &p );
	status->CurSpeed = pop_int32( &p );
	status->uCurSpeed = pop_int16( &p );
	status->Ipwr = pop_int16( &p );
	status->Upwr = pop_int16( &p );
	status->Iusb = pop_int16( &p );
	status->Uusb = pop_int16( &p );
	status->CurT = pop_int16( &p );
	status->Flags = pop_uint32( &p );
	status->GPIOFlags = pop_uint32( &p );
	status->CmdBufFreeSpace = pop_uint8( &p );
	pop_garbage( &p, 4 );

	return check_in_overrun( id, p-response, sizeof(response), response );
}

result_t get_status_impl_calb (device_t id, status_calb_t* status_calb, const calibration_t* calibration)
{
	result_t result;
	status_t inner;

	if ((result = get_status_impl(id, &inner)) != result_ok)
		return result;

	status_calb->MoveSts = inner.MoveSts;
	status_calb->MvCmdSts = inner.MvCmdSts;
	status_calb->PWRSts = inner.PWRSts;
	status_calb->EncSts = inner.EncSts;
	status_calb->WindSts = inner.WindSts;
	XI_normal_to_calibrate(status_calb->CurPosition, inner.CurPosition, inner.uCurPosition, calibration);
	status_calb->EncPosition = inner.EncPosition;
	XI_normal_to_calibrate(status_calb->CurSpeed, inner.CurSpeed, inner.uCurSpeed, calibration);
	status_calb->Ipwr = inner.Ipwr;
	status_calb->Upwr = inner.Upwr;
	status_calb->Iusb = inner.Iusb;
	status_calb->Uusb = inner.Uusb;
	status_calb->CurT = inner.CurT;
	status_calb->Flags = inner.Flags;
	status_calb->GPIOFlags = inner.GPIOFlags;
	status_calb->CmdBufFreeSpace = inner.CmdBufFreeSpace;

	return result;
}

result_t XIMC_API get_chart_data (device_t id, chart_data_t* chart_data)
{
	result_t result;
	byte response[38], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "getc", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	chart_data->WindingVoltageA = pop_int16( &p );
	chart_data->WindingVoltageB = pop_int16( &p );
	chart_data->WindingVoltageC = pop_int16( &p );
	chart_data->WindingCurrentA = pop_int16( &p );
	chart_data->WindingCurrentB = pop_int16( &p );
	chart_data->WindingCurrentC = pop_int16( &p );
	chart_data->Pot = pop_uint16( &p );
	chart_data->Joy = pop_uint16( &p );
	chart_data->DutyCycle = pop_int16( &p );
	pop_garbage( &p, 14 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t get_device_information_impl_unsynced (device_t id, device_information_t* device_information)
{
	result_t result;
	byte response[36], *p  = response;

	if ((result = command_checked_str_unsynced( id, "geti", response, sizeof(response) )) != result_ok)
		return result;
	p += 4;

	pop_str( &p, device_information->Manufacturer, 4 );
	pop_str( &p, device_information->ManufacturerId, 2 );
	pop_str( &p, device_information->ProductDescription, 8 );
	pop_garbage( &p, 16 );

	return check_in_overrun( id, p-response, sizeof(response), response );
}

result_t get_device_information_impl (device_t id, device_information_t* device_information)
{
	result_t result;
	byte response[36], *p  = response;

	if ((result = command_checked_str( id, "geti", response, sizeof(response) )) != result_ok)
		return result;
	p += 4;

	pop_str( &p, device_information->Manufacturer, 4 );
	pop_str( &p, device_information->ManufacturerId, 2 );
	pop_str( &p, device_information->ProductDescription, 8 );
	pop_garbage( &p, 16 );

	return check_in_overrun( id, p-response, sizeof(response), response );
}

result_t XIMC_API get_serial_number (device_t id, unsigned int* SerialNumber)
{
	result_t result;
	byte response[10], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gser", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	*SerialNumber = pop_uint32( &p );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_firmware_version (device_t id, unsigned int* Major, unsigned int* Minor, unsigned int* Release)
{
	result_t result;
	byte response[10], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gfwv", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	*Major = pop_uint8( &p );
	*Minor = pop_uint8( &p );
	*Release = pop_uint16( &p );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API service_command_updf (device_t id)
{
	return command_checked_echo_str( id, "updf" );
}


result_t XIMC_API set_serial_number (device_t id, const serial_number_t* serial_number)
{
	result_t result;
	byte command[50], *p  = command;
	unsigned int i;

	lock( id );

	push_str( &p, "sser" );
	push_uint32( &p, serial_number->SN );
	for (i = 0; i < 32; ++i)
		push_uint8( &p, serial_number->Key[i] );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_analog_data (device_t id, analog_data_t* analog_data)
{
	result_t result;
	byte response[76], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "rdan", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	analog_data->A1Voltage_ADC = pop_uint16( &p );
	analog_data->A2Voltage_ADC = pop_uint16( &p );
	analog_data->B1Voltage_ADC = pop_uint16( &p );
	analog_data->B2Voltage_ADC = pop_uint16( &p );
	analog_data->SupVoltage_ADC = pop_uint16( &p );
	analog_data->ACurrent_ADC = pop_uint16( &p );
	analog_data->BCurrent_ADC = pop_uint16( &p );
	analog_data->FullCurrent_ADC = pop_uint16( &p );
	analog_data->Temp_ADC = pop_uint16( &p );
	analog_data->Joy_ADC = pop_uint16( &p );
	analog_data->Pot_ADC = pop_uint16( &p );
	analog_data->L5_ADC = pop_uint16( &p );
	analog_data->H5_ADC = pop_uint16( &p );
	analog_data->A1Voltage = pop_int16( &p );
	analog_data->A2Voltage = pop_int16( &p );
	analog_data->B1Voltage = pop_int16( &p );
	analog_data->B2Voltage = pop_int16( &p );
	analog_data->SupVoltage = pop_int16( &p );
	analog_data->ACurrent = pop_int16( &p );
	analog_data->BCurrent = pop_int16( &p );
	analog_data->FullCurrent = pop_int16( &p );
	analog_data->Temp = pop_int16( &p );
	analog_data->Joy = pop_int16( &p );
	analog_data->Pot = pop_int16( &p );
	analog_data->L5 = pop_int16( &p );
	analog_data->H5 = pop_int16( &p );
	analog_data->deprecated = pop_uint16( &p );
	analog_data->R = pop_int32( &p );
	analog_data->L = pop_int32( &p );
	pop_garbage( &p, 8 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_debug_read (device_t id, debug_read_t* debug_read)
{
	result_t result;
	byte response[142], *p  = response;
	unsigned int i;

	lock( id );

	if ((result = command_checked_str( id, "dbgr", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	for (i = 0; i < 128; ++i)
		debug_read->DebugData[i] = pop_uint8( &p );
	pop_garbage( &p, 8 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t service_command_clear_fram_impl (device_t id)
{
	return command_checked_str( id, "clfr", NULL, 0 );
}


result_t XIMC_API set_stage_name (device_t id, const stage_name_t* stage_name)
{
	result_t result;
	byte command[30], *p  = command;

	lock( id );

	push_str( &p, "snme" );
	push_data( &p, stage_name->PositionerName, 16 );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_stage_name (device_t id, stage_name_t* stage_name)
{
	result_t result;
	byte response[30], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gnme", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, stage_name->PositionerName, 16 );
	pop_garbage( &p, 8 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_stage_information (device_t id, const stage_information_t* stage_information)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "ssti" );
	push_data( &p, stage_information->Manufacturer, 16 );
	push_data( &p, stage_information->PartNumber, 24 );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_stage_information (device_t id, stage_information_t* stage_information)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gsti", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, stage_information->Manufacturer, 16 );
	pop_str( &p, stage_information->PartNumber, 24 );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_stage_settings (device_t id, const stage_settings_t* stage_settings)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "ssts" );
	push_float( &p, stage_settings->LeadScrewPitch );
	push_data( &p, stage_settings->Units, 8 );
	push_float( &p, stage_settings->MaxSpeed );
	push_float( &p, stage_settings->TravelRange );
	push_float( &p, stage_settings->SupplyVoltageMin );
	push_float( &p, stage_settings->SupplyVoltageMax );
	push_float( &p, stage_settings->MaxCurrentConsumption );
	push_float( &p, stage_settings->HorizontalLoadCapacity );
	push_float( &p, stage_settings->VerticalLoadCapacity );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_stage_settings (device_t id, stage_settings_t* stage_settings)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gsts", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	stage_settings->LeadScrewPitch = pop_float( &p );
	pop_str( &p, stage_settings->Units, 8 );
	stage_settings->MaxSpeed = pop_float( &p );
	stage_settings->TravelRange = pop_float( &p );
	stage_settings->SupplyVoltageMin = pop_float( &p );
	stage_settings->SupplyVoltageMax = pop_float( &p );
	stage_settings->MaxCurrentConsumption = pop_float( &p );
	stage_settings->HorizontalLoadCapacity = pop_float( &p );
	stage_settings->VerticalLoadCapacity = pop_float( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_motor_information (device_t id, const motor_information_t* motor_information)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "smti" );
	push_data( &p, motor_information->Manufacturer, 16 );
	push_data( &p, motor_information->PartNumber, 24 );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_motor_information (device_t id, motor_information_t* motor_information)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gmti", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, motor_information->Manufacturer, 16 );
	pop_str( &p, motor_information->PartNumber, 24 );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_motor_settings (device_t id, const motor_settings_t* motor_settings)
{
	result_t result;
	byte command[112], *p  = command;

	lock( id );

	push_str( &p, "smts" );
	push_uint8( &p, motor_settings->MotorType );
	push_uint8( &p, motor_settings->ReservedField );
	push_uint16( &p, motor_settings->Poles );
	push_uint16( &p, motor_settings->Phases );
	push_float( &p, motor_settings->NominalVoltage );
	push_float( &p, motor_settings->NominalCurrent );
	push_float( &p, motor_settings->NominalSpeed );
	push_float( &p, motor_settings->NominalTorque );
	push_float( &p, motor_settings->NominalPower );
	push_float( &p, motor_settings->WindingResistance );
	push_float( &p, motor_settings->WindingInductance );
	push_float( &p, motor_settings->RotorInertia );
	push_float( &p, motor_settings->StallTorque );
	push_float( &p, motor_settings->DetentTorque );
	push_float( &p, motor_settings->TorqueConstant );
	push_float( &p, motor_settings->SpeedConstant );
	push_float( &p, motor_settings->SpeedTorqueGradient );
	push_float( &p, motor_settings->MechanicalTimeConstant );
	push_float( &p, motor_settings->MaxSpeed );
	push_float( &p, motor_settings->MaxCurrent );
	push_float( &p, motor_settings->MaxCurrentTime );
	push_float( &p, motor_settings->NoLoadCurrent );
	push_float( &p, motor_settings->NoLoadSpeed );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_motor_settings (device_t id, motor_settings_t* motor_settings)
{
	result_t result;
	byte response[112], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gmts", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	motor_settings->MotorType = pop_uint8( &p );
	motor_settings->ReservedField = pop_uint8( &p );
	motor_settings->Poles = pop_uint16( &p );
	motor_settings->Phases = pop_uint16( &p );
	motor_settings->NominalVoltage = pop_float( &p );
	motor_settings->NominalCurrent = pop_float( &p );
	motor_settings->NominalSpeed = pop_float( &p );
	motor_settings->NominalTorque = pop_float( &p );
	motor_settings->NominalPower = pop_float( &p );
	motor_settings->WindingResistance = pop_float( &p );
	motor_settings->WindingInductance = pop_float( &p );
	motor_settings->RotorInertia = pop_float( &p );
	motor_settings->StallTorque = pop_float( &p );
	motor_settings->DetentTorque = pop_float( &p );
	motor_settings->TorqueConstant = pop_float( &p );
	motor_settings->SpeedConstant = pop_float( &p );
	motor_settings->SpeedTorqueGradient = pop_float( &p );
	motor_settings->MechanicalTimeConstant = pop_float( &p );
	motor_settings->MaxSpeed = pop_float( &p );
	motor_settings->MaxCurrent = pop_float( &p );
	motor_settings->MaxCurrentTime = pop_float( &p );
	motor_settings->NoLoadCurrent = pop_float( &p );
	motor_settings->NoLoadSpeed = pop_float( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_encoder_information (device_t id, const encoder_information_t* encoder_information)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "seni" );
	push_data( &p, encoder_information->Manufacturer, 16 );
	push_data( &p, encoder_information->PartNumber, 24 );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_encoder_information (device_t id, encoder_information_t* encoder_information)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "geni", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, encoder_information->Manufacturer, 16 );
	pop_str( &p, encoder_information->PartNumber, 24 );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_encoder_settings (device_t id, const encoder_settings_t* encoder_settings)
{
	result_t result;
	byte command[54], *p  = command;

	lock( id );

	push_str( &p, "sens" );
	push_float( &p, encoder_settings->MaxOperatingFrequency );
	push_float( &p, encoder_settings->SupplyVoltageMin );
	push_float( &p, encoder_settings->SupplyVoltageMax );
	push_float( &p, encoder_settings->MaxCurrentConsumption );
	push_uint32( &p, encoder_settings->PPR );
	push_uint32( &p, encoder_settings->EncoderSettings );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_encoder_settings (device_t id, encoder_settings_t* encoder_settings)
{
	result_t result;
	byte response[54], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gens", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	encoder_settings->MaxOperatingFrequency = pop_float( &p );
	encoder_settings->SupplyVoltageMin = pop_float( &p );
	encoder_settings->SupplyVoltageMax = pop_float( &p );
	encoder_settings->MaxCurrentConsumption = pop_float( &p );
	encoder_settings->PPR = pop_uint32( &p );
	encoder_settings->EncoderSettings = pop_uint32( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_hallsensor_information (device_t id, const hallsensor_information_t* hallsensor_information)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "shsi" );
	push_data( &p, hallsensor_information->Manufacturer, 16 );
	push_data( &p, hallsensor_information->PartNumber, 24 );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_hallsensor_information (device_t id, hallsensor_information_t* hallsensor_information)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "ghsi", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, hallsensor_information->Manufacturer, 16 );
	pop_str( &p, hallsensor_information->PartNumber, 24 );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_hallsensor_settings (device_t id, const hallsensor_settings_t* hallsensor_settings)
{
	result_t result;
	byte command[50], *p  = command;

	lock( id );

	push_str( &p, "shss" );
	push_float( &p, hallsensor_settings->MaxOperatingFrequency );
	push_float( &p, hallsensor_settings->SupplyVoltageMin );
	push_float( &p, hallsensor_settings->SupplyVoltageMax );
	push_float( &p, hallsensor_settings->MaxCurrentConsumption );
	push_uint32( &p, hallsensor_settings->PPR );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_hallsensor_settings (device_t id, hallsensor_settings_t* hallsensor_settings)
{
	result_t result;
	byte response[50], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "ghss", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	hallsensor_settings->MaxOperatingFrequency = pop_float( &p );
	hallsensor_settings->SupplyVoltageMin = pop_float( &p );
	hallsensor_settings->SupplyVoltageMax = pop_float( &p );
	hallsensor_settings->MaxCurrentConsumption = pop_float( &p );
	hallsensor_settings->PPR = pop_uint32( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_gear_information (device_t id, const gear_information_t* gear_information)
{
	result_t result;
	byte command[70], *p  = command;

	lock( id );

	push_str( &p, "sgri" );
	push_data( &p, gear_information->Manufacturer, 16 );
	push_data( &p, gear_information->PartNumber, 24 );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_gear_information (device_t id, gear_information_t* gear_information)
{
	result_t result;
	byte response[70], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "ggri", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, gear_information->Manufacturer, 16 );
	pop_str( &p, gear_information->PartNumber, 24 );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_gear_settings (device_t id, const gear_settings_t* gear_settings)
{
	result_t result;
	byte command[58], *p  = command;

	lock( id );

	push_str( &p, "sgrs" );
	push_float( &p, gear_settings->ReductionIn );
	push_float( &p, gear_settings->ReductionOut );
	push_float( &p, gear_settings->RatedInputTorque );
	push_float( &p, gear_settings->RatedInputSpeed );
	push_float( &p, gear_settings->MaxOutputBacklash );
	push_float( &p, gear_settings->InputInertia );
	push_float( &p, gear_settings->Efficiency );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_gear_settings (device_t id, gear_settings_t* gear_settings)
{
	result_t result;
	byte response[58], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "ggrs", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	gear_settings->ReductionIn = pop_float( &p );
	gear_settings->ReductionOut = pop_float( &p );
	gear_settings->RatedInputTorque = pop_float( &p );
	gear_settings->RatedInputSpeed = pop_float( &p );
	gear_settings->MaxOutputBacklash = pop_float( &p );
	gear_settings->InputInertia = pop_float( &p );
	gear_settings->Efficiency = pop_float( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API set_accessories_settings (device_t id, const accessories_settings_t* accessories_settings)
{
	result_t result;
	byte command[114], *p  = command;

	lock( id );

	push_str( &p, "sacc" );
	push_data( &p, accessories_settings->MagneticBrakeInfo, 24 );
	push_float( &p, accessories_settings->MBRatedVoltage );
	push_float( &p, accessories_settings->MBRatedCurrent );
	push_float( &p, accessories_settings->MBTorque );
	push_uint32( &p, accessories_settings->MBSettings );
	push_data( &p, accessories_settings->TemperatureSensorInfo, 24 );
	push_float( &p, accessories_settings->TSMin );
	push_float( &p, accessories_settings->TSMax );
	push_float( &p, accessories_settings->TSGrad );
	push_uint32( &p, accessories_settings->TSSettings );
	push_uint32( &p, accessories_settings->LimitSwitchesSettings );
	push_garbage( &p, 24 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return unlocker( id, result );

	return unlocker( id, command_checked_echo( id, command, sizeof(command)) );
}

result_t XIMC_API get_accessories_settings (device_t id, accessories_settings_t* accessories_settings)
{
	result_t result;
	byte response[114], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gacc", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	pop_str( &p, accessories_settings->MagneticBrakeInfo, 24 );
	accessories_settings->MBRatedVoltage = pop_float( &p );
	accessories_settings->MBRatedCurrent = pop_float( &p );
	accessories_settings->MBTorque = pop_float( &p );
	accessories_settings->MBSettings = pop_uint32( &p );
	pop_str( &p, accessories_settings->TemperatureSensorInfo, 24 );
	accessories_settings->TSMin = pop_float( &p );
	accessories_settings->TSMax = pop_float( &p );
	accessories_settings->TSGrad = pop_float( &p );
	accessories_settings->TSSettings = pop_uint32( &p );
	accessories_settings->LimitSwitchesSettings = pop_uint32( &p );
	pop_garbage( &p, 24 );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t XIMC_API get_bootloader_version (device_t id, unsigned int* Major, unsigned int* Minor, unsigned int* Release)
{
	result_t result;
	byte response[10], *p  = response;

	lock( id );

	if ((result = command_checked_str( id, "gblv", response, sizeof(response) )) != result_ok)
		return unlocker( id, result );
	p += 4;

	*Major = pop_uint8( &p );
	*Minor = pop_uint8( &p );
	*Release = pop_uint16( &p );

	return unlocker( id, check_in_overrun( id, p-response, sizeof(response), response ) );
}

result_t service_command_goto_firmware_impl (device_t id, service_result_t* sresult)
{
	result_t result;
	byte command[4];
	byte response[15];
	byte* p = command;

	push_str( &p, "gofw" );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked_str( id, "gofw", response, sizeof(response) )) != result_ok)
		return result;

	p = response;
	p += 4;
	*sresult = pop_uint8( &p );
	pop_garbage( &p, 8 );

	return check_in_overrun_without_crc( id, p-response, sizeof(response), response );
}

result_t service_command_has_firmware_impl (device_t id, service_result_t* sresult)
{
	result_t result;
	byte command[4];
	byte response[15];
	byte* p = command;

	push_str( &p, "hasf" );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked_str( id, "hasf", response, sizeof(response) )) != result_ok)
		return result;

	p = response;
	p += 4;
	*sresult = pop_uint8( &p );
	pop_garbage( &p, 8 );

	return check_in_overrun_without_crc( id, p-response, sizeof(response), response );
}

result_t service_command_write_key_impl (device_t id, const uint8_t* Key)
{
	result_t result;
	service_result_t sresult;
	byte command[46];
	byte response[15];
	byte* p = command;
	unsigned int i;

	push_str( &p, "wkey" );
	for (i = 0; i < 32; ++i)
		push_uint8( &p, Key[i] );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked( id, command, sizeof(command), response, sizeof(response) )) != result_ok)
		return result;

	p = response;
	p += 4;
	sresult = pop_uint8( &p );
	pop_garbage( &p, 8 );

	if ((result = check_in_overrun( id, p-response, sizeof(response), response )) != result_ok)
		return result_error;
	
	switch (sresult)
	{
		case RESULT_OK: 
			result = result_ok;
			break;
		default: 
			dump_service_result( L"service_command_write_key_impl", sresult );
			result = result_error;
			break;
	}

	return result;
}

result_t service_command_connect_impl (device_t id)
{
	result_t result;
	service_result_t sresult;
	byte command[14];
	byte response[15];
	byte* p = command;

	push_str( &p, "conn" );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked( id, command, sizeof(command), response, sizeof(response) )) != result_ok)
		return result;

	p = response;
	p += 4;
	sresult = pop_uint8( &p );
	pop_garbage( &p, 8 );

	if ((result = check_in_overrun( id, p-response, sizeof(response), response )) != result_ok)
		return result_error;
	
	switch (sresult)
	{
		case RESULT_OK: 
			result = result_ok;
			break;
		default: 
			dump_service_result( L"service_command_connect_impl", sresult );
			result = result_error;
			break;
	}

	return result;
}

result_t service_command_disconnect_impl (device_t id)
{
	result_t result;
	service_result_t sresult;
	byte command[14];
	byte response[15];
	byte* p = command;

	push_str( &p, "disc" );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked( id, command, sizeof(command), response, sizeof(response) )) != result_ok)
		return result;

	p = response;
	p += 4;
	sresult = pop_uint8( &p );
	pop_garbage( &p, 8 );

	if ((result = check_in_overrun( id, p-response, sizeof(response), response )) != result_ok)
		return result_error;
	
	switch (sresult)
	{
		case RESULT_OK: 
			result = result_ok;
			break;
		default: 
			dump_service_result( L"service_command_disconnect_impl", sresult );
			result = result_error;
			break;
	}

	return result;
}

result_t service_command_write_data_impl (device_t id, const uint8_t* Data)
{
	result_t result;
	byte command[142], *p  = command;
	unsigned int i;

	push_str( &p, "wdat" );
	for (i = 0; i < 128; ++i)
		push_uint8( &p, Data[i] );
	push_garbage( &p, 8 );
	push_crc( &p, command, p-command );

	if ((result = check_out_overrun( p-command, sizeof(command) )) != result_ok)
		return result;

	if ((result = command_checked_echo( id, command, sizeof(command))) != result_ok)
		return result;

	return result;
}

result_t service_command_reset_impl (device_t id)
{
	return command_checked_str( id, "rest", NULL, 0 );
}



/*
 -------------------------
   END OF GENERATED CODE
 -------------------------
*/

#if defined(__cplusplus)
};
#endif

// vim: syntax=c tabstop=4 shiftwidth=4

